package Database;
#
# Very basic simulation of database interface for OAI v1.0 tutorial
# Simeon Warner - May2001...
#
# [CVS: $Id: Database.pm,v 1.4 2001/06/19 17:28:43 simeon Exp $]
use strict;
use Carp;

use vars qw(@ISA @EXPORT);
use Exporter;
@ISA=qw(Exporter);
@EXPORT=qw(&recordExists &recordIsDeleted &getDatestamp &getMetadata 
           &getMetadataFormats &getMetadataFormatInfo &getIdsByDate);

# Simple structure to simulate database
# 
my %items = (
  'record1' => [ '1999-01-11', {
    'oai_dc' => ['Title','Item 1', 'Creator','A Person'],
    'wibble' => ['title','Item 1', 'wibble','Wibble property'] } ] ,
  'record2' => [ '1999-02-12', {
    'oai_dc' => ['Title','Item 2', 'Creator','A N Other'] } ] , 
  'record3' => [ '2000-03-13', undef ]  #deleted
);

# Namespace information for particular metadata formats
#
my %metadataFormatInfo = (
  'oai_dc' => [ 'http://www.openarchives.org/OAI/dc.xsd',
                { 'xmlns' => 'http://purl.org/dc/elements/1.1/',
                  'xmlns:xsi' => 'http://www.w3.org/2000/10/XMLSchema-instance',
                  'xsi:schemaLocation' => 'http://purl.org/dc/elements/1.1/ http://www.openarchives.org/OAI/dc.xsd' } ],
  'wibble' => [ 'http://wibble.org/wibble.xsd',
                { 'xmlns' => 'http://purl.org/dc/elements/1.1/',
                  'xmlns:xsi' => 'http://www.w3.org/2000/10/XMLSchema-instance',
                  'xsi:schemaLocation' => 'http://purl.org/dc/elements/1.1/ http://wibble.org/wibble.xsd' }]

);

# True if item exists (current or deleted) in database 
#
sub recordExists($) {
  my $id=shift;
  return(defined($items{$id}));
}

# True if item exists and has 'deleted' status in the database
#
sub recordIsDeleted($) {
  my $id=shift;
  return(&recordExists($id) and !defined($items{$id}->[1]));
}


# Returns datestamp for specified item, undef if item not known
#
sub getDatestamp($) {
  my ($id)=@_;
  return(undef) unless (&recordExists($id));
  return($items{$id}->[0]);
}


# Returns metadata as an array of (element, value) pairs for requested 
# format if available, undef otherwise
# 
sub getMetadata($$) {
  my ($id,$metaformat)=@_;
  return(undef) unless (&recordExists($id) and !&recordIsDeleted($id));
  return($items{$id}->[1]{$metaformat});
}
 

# Returns metadata formats available and corresponding XML namespace 
# information for either a particular item or for the whole database.
# Formats for whole database are found by looking at every item, 
# clearly a crazy way to do this for anything but a very small database!
# 
sub getMetadataFormats($) {
  my ($id)=@_;
  my %mfs=();
  my @ids=( defined($id) ? ($id) : &getIdsByDate() );
  foreach $id (@ids) {
    foreach my $mf (&metaFormatsForRecord($id)) {
      $mfs{$mf}=1;
    }
  } 
  return(keys %mfs);
}

# Returns namespace URL and pointer to attributes hash for element
# declaration for the specified metadataPrefix 
#
sub getMetadataFormatInfo($) {
  my $mf = shift;
  return(@{$metadataFormatInfo{$mf}});
}

# Returns an array of ids for items that have a datestamp between the
# specified from and until dates (inclusive). If from is not specified
# then we start from the beginning of time; if until is not specified
# then we continue to the end of time. 
#
sub getIdsByDate {
  my ($from,$until)=@_;
  my $dsf=(defined($from)?&datestampToNum($from):0);  
  my $dsu=(defined($until)?&datestampToNum($until):99999999);  
  my @ids=();
  foreach my $id (keys %items) {
    my $ds=&datestampToNum(&getDatestamp($id));  
    next unless ($ds>=$dsf and $ds<=$dsu);
    push(@ids,$id);
  }
  return(@ids);
}


# Simply removes hypens from datestamp to allow numerical comparison
#
sub datestampToNum($) {
  my ($d)=@_;
  $d=~s/\-//g; 
  return($d);
}


# Returns a list of metadata formats for a record 
#
sub metaFormatsForRecord($) {
  my ($id)=@_;
  return() unless (&recordExists($id) and !&recordIsDeleted($id));
  return(keys %{$items{$id}[1]});
}  

1;














