#!/usr/bin/perl
#
# Simple CGI handler for OAI version 1 requests
#
# Accepts requests by either GET or POST and deals with any
# other web-server specific parts of the server. 
#
# Simeon Warner - May2001...
#
# [CVS: $Id: oai1.pl,v 1.2 2001/06/05 16:07:56 simeon Exp $]
use strict;

use Getopt::Std;
use vars qw($opt_r);
getopts('r:') || die "Bad arguments";
 
use CGI;
use OAIServer qw(&OAICheckRequest &OAISatisfyRequest);


# Create query object and parse both GET and POST data
# (take from command line if -r flag specified)
my $query = new CGI($opt_r);

# Read all arguments into %args hash, just take first if any are 
# multiply defined (eg. name1=val1&name1=val2). Perhaps this
# should be an error but it is neatest just to take the first.
#
my %args=();
foreach my $name ($query->param) {
  my @values = $query->param($name);
  if (scalar(@values)==0) {
    $args{$name}='';
  } else {
    $args{$name}=$values[0];
  }
}

# Now deal with the request... 
#
my $CRLF = "\r\n"; #for HTTP replies
my $err = &OAICheckRequest(\%args);
if ($err) {
  print "Status: 400 Malformed request$CRLF";
  print "Content-Type: text/plain$CRLF$CRLF$err\n";
} else {
  $|=1; #set autoflush for STDOUT
  print "Content-Type: text/xml$CRLF$CRLF";
  &OAISatisfyRequest(\%args);
}
exit(0);

###END###


