#!/usr/bin/perl
#
# Test code for OAIGet module
# Harvests all identifiers from an archive
#
# Simeon Warner - 8Nov2000...
# [CVS: $Id: oaiharvest.pl,v 1.5 2001/06/07 17:21:39 simeon Exp $]
use strict;

use OAIGet;
use OAIParser;

my $DEFAULT_SLEEP=1;
my $contact=''; ### PUT YOUR EMAIL ADDRESS HERE ###
unless ($contact) { die "Please configure contact address variable"; }

my ($myname)=__FILE__=~/([^\/]+)$/;
my ($userAgent)='$Revision: 1.5 $'=~/([\d.]+)/;
$userAgent="$myname-$userAgent";

use Getopt::Std;
use vars qw($opt_f $opt_g $opt_d $opt_h $opt_s $opt_w $opt_r $opt_m 
            $opt_l $opt_R $opt_i);
(getopts('f:gd:s:w:rm:lR:i:h') and !$opt_h) || &usage();
&usage() if (scalar(@ARGV)==0);

my $method=($opt_g?"GET":"POST");
my $outDir=($opt_d?$opt_d:'.');
my $sleep=($opt_w?$opt_w:$DEFAULT_SLEEP);
my $setSpec=($opt_s?$opt_s:undef);
my $listRecords=$opt_r;
my $metadataPrefix=($opt_m?$opt_m:'oai_dc');
my $fromDate=($opt_f?$opt_f:undef);
my $identifyFile=$opt_i;

my $lax=$opt_l;
my $resume=$opt_R;

$|=1; #autoflush for stdout

die "$myname: Must be one baseURL as argument." if (scalar(@ARGV)!=1); 
my $url=$ARGV[0];

# Check output directory exists and is writable
#
unless (-d $outDir and -w $outDir ) {
  print  "$myname: Problem with output dir, aborting\n";
  exit(1);
}

{
  print "\n$myname: Harvest from $url using $method\n";
  #
  unless ($url=~m%^http://([\w\d\.\-:]+)/([\w\d\.\-_\/]*)$%) {
    print "$myname: Don't understand URL, aborting\n";
    exit(1);
  }
  #
  #
  # Do Identify request
  #
  if (!$resume) { 
    my $file="$outDir/Identify";
    my %args=('verb'=>'Identify');
    my ($err,$msg,$content)=&OAIGet($url,\%args,$method,$contact,$userAgent,0,1);
    if ($err!=200) {
      print "$myname: Error: $err\nError description: $msg\nMessage: $content";
      print "$myname: Aborting.\n";
      exit(2);
    } else {
      open(OUT,">$file") || die "Can't open output file, $file.";
      print OUT $content;
      close OUT;
    }
    sleep($sleep); #just in case something silly is going on
    #
    # Compare with existing Identify if specified
    #
    if ($identifyFile) {
      my $diff=`diff $identifyFile $file`;
      # 
      # Check that the difference is in nothing but the responseDate
      if (scalar(split('\n',$diff))>4) {
        print "$myname: Identify response differs by more than date stamp, aborting!\n";   
        exit(3);
      } elsif ($diff!~/responseDate/) {
        print "$myname: Identify response doesn't differ in datestamp??, aborting!\n";
        exit(3);    
      } else {
        print "$myname: Identify response unchanged from reference (except date)\n"; 
      }             
    } 
  }
  #
  # Work out if we want to do a complete harvest, or a harvest from
  # a particular date
  # 
  if (!$resume) {
    if (defined($fromDate)) {
      print "$myname: Incremental harvest from $fromDate\n";  
    } elsif (defined($identifyFile)) {
      print "$myname: Reading $identifyFile to get from date\n";
      my $parser=new OAIParser();
      my $err=$parser->parseFile($identifyFile);
      my ($rd,$from)=$parser->getResponseDate();
      if (defined($rd)) {
        $fromDate=$from;
        print "$myname: Incremenetal harvest from $fromDate (from $identifyFile)\n";  
      } else {
        print "$myname: Can't get from date from $identifyFile: $from\n";
        next;
      }
    } else {
      print "$myname: Doing complete harvest.\n"; 
    }
  }  
  #
  # Do ListIdentifiers/Records request
  #
  { 
    my $totalIds=0;
    my %args=();
    my $outFile='no_output_file_defined';
    my $verb='no_verb_defined';
    if ($listRecords) {
      $outFile="$outDir/ListRecords";
      $verb='ListRecords';
    } else {
      $outFile="$outDir/ListIdentifiers";
      $verb='ListIdentifiers';
    }
    if ($resume) {
      $args{'resumptionToken'}=$resume;
    } else {
      $args{'metadataPrefix'}=$metadataPrefix if ($listRecords);
      $args{'set'}=$setSpec if ($setSpec);
      $args{'from'}=$fromDate if ($fromDate);
    }
    $args{'verb'}=$verb;
    #
    my $requestNum=0;
    #
    while (scalar(keys %args)>0) {
      $requestNum++; 
      my ($err,$msg,$content)=&OAIGet($url,\%args,$method,$contact,$userAgent,0,1);
      if ($err!=200) {
        print "$myname: Error: $err\nError description: $msg\nMessage: $content";
        print "$myname: Aborting!\n";
        %args=();
        next;
      } else {
        my $file="$outFile.$requestNum";
        open(OUT,">$file") || die "Can't open output file, $file,";
        print OUT $content;
        close OUT;
        #
        # Read in and parse response
        #
        my $parser=new OAIParser();
        my $err=$parser->parse($content);
        if ($err) {
          print "$myname: Error - $err\n";
          print "$myname: Failed to parse response, aborting!\n";
          exit(4);  
        }
        #
        # Count number of records returned
        #
        my $numId=0; 
        if ($listRecords) {
          $numId=$parser->find(['ListRecords','record'],-1);   
        } else {
          $numId=$parser->find(['ListIdentifiers','identifier'],-1);   
        }
        $totalIds+=$numId;
        print "$myname: Got $numId OAI identifiers (running total: $totalIds)\n";
        #
        # Now was this partial or complete?
        #
        my ($rt,$err)=$parser->getResumptionToken();
        if (defined($rt)) {
          print "$myname: Got resumptionToken: `$rt'\n";
          if ($lax and ($rt=~s/\s+$//)) {
            print "$myname: Stripped trailing whitespace from resumptionToken: `$rt'\n";
          }
          %args=('verb'=>$verb,'resumptionToken'=>$rt);
        } elsif (defined($err)) {
          print "$myname: Error extracting resumptionToken from reply `$err', aborting!\n";
          %args=();
        } else {
          print "$myname: No resumptionToken, request complete.\n";
          %args=();
        }

      } 
      sleep($sleep); #just in case something silly is going on
    }
  }
}
print "$myname: Done.\n";


sub usage {
  die("usage: $myname [-h] [-g] [-b dir] scriptURL [scriptURL2...]
   Does harvest using POST request for scriptURL
   -g      GET instead of POST
   -r      do ListRecords instead of ListIdentifiers
   -m metadaaPrefix   use given metadataPrefix with -r    
   -s setSpec         use gievn setSpec with ListIdentifiers/ListRecords
   -R resumptionToken resume from this resumptiontoken
   -d dir  use dir for output instead of pwd 
   -w secs wait (sleep) for given time between requests instead of default ${DEFAULT_SLEEP}s  
   -l      lax mode (strips any trailing whitespace from resumptionToken)
   -h      this help.\n");
}
